/*  
 * Core of this code is based on work of
 */
/*
 *  DLA - Crystal Simulation 
 *
 *  More Info: http://stungeye.com/archives/2007/10/usually_permane.php
 *
 *  Wally Glutton 2007
 *  http://stungeye.com
 *
 *  License: http://creativecommons.org/licenses/by-sa/3.0/
 *
 */
 

class Particle {
  
  // some colors
  color white = color(80, 125, 42);
  color sapGreen = color (80, 125, 42);
  color gray = color(125, 125, 125);
  color black = color(0, 0, 0);
  
  // Particle properties
  float positionX;
  float positionY;
  PVector velocityVector;
  PVector gravityVector;
  float life;
  int state;
  int sourceID;
  int particleID;
  
  // construct a particle
  Particle(float particleOriginX, float particleOriginY, PVector velocityVector, float gravity, int life, int sourceID, int particleID) {
    this.positionX = particleOriginX;
    this.positionY = particleOriginY;
    this.velocityVector = PVector.mult(velocityVector, random(0, 1));
    this.gravityVector = PVector.mult(new PVector(0, gravity / 2), random(0, 1));
    this.life = life * random(1, 5);
    this.state = 0;
    this.sourceID = sourceID;
    this.particleID = particleID;
  } 
  
  void setID(int particleID) {
    this.particleID = particleID;
  }
  
  // check if particle is out of screen boundary
  boolean outOfBoundary(float x, float y) {
    if ((round(x) < 0) || (round(x) > width)) return true;
    if ((round(y) < 0) || (round(y) > 4 * height / 5 - 2)) return true;
    return false;
  }
  
  // control movement of particle
  void moveParticle() {
    ((ScreenPixel) allPixels.get(round(this.positionY) * width + round(this.positionX))).setSourceID(-1);
    ((ScreenPixel) allPixels.get(round(this.positionY) * width + round(this.positionX))).setParticleID(-1);
    this.velocityVector.add(gravityVector);
    this.positionX += velocityVector.x + random(-2,2);
    this.positionY += velocityVector.y + random(-2,2);
    this.life--;
    // remove particle if its out of boundary
    if (outOfBoundary(this.positionX, this.positionY)) {
      this.life = 0;
      return;
    }
    ((ScreenPixel) allPixels.get(round(this.positionY) * width + round(this.positionX))).setSourceID(this.sourceID);
    ((ScreenPixel) allPixels.get(round(this.positionY) * width + round(this.positionX))).setParticleID(this.particleID);
  }
}
