import processing.core.*; 
import processing.xml.*; 

import java.applet.*; 
import java.awt.*; 
import java.awt.image.*; 
import java.awt.event.*; 
import java.io.*; 
import java.net.*; 
import java.text.*; 
import java.util.*; 
import java.util.zip.*; 
import java.util.regex.*; 

public class dlaplaygroundoptimized extends PApplet {

// user interface variables
PFont font = createFont("Showcard Gothic", 20);
int ribbonHeight;
int buttonWidth;
int buttonHeight;
int scrollLoose = 10;
int margin;

// DLA playground variables
ArrayList sources;
DLA_Source currentSource;
boolean isNewGrow;
ArrayList allPixels;
int sourceID;

// program modes
final static int PLANT_MODE = 0;
final static int SOURCE_MODE = 1;
final static int GROW_MODE = 2;
final static int STOP_MODE = 3;
int mode = 3;

// particle system source variables
int pointSourceOriginX;
int pointSourceOriginY;
int lineSourceBeginX;
int lineSourceBeginY;
int lineSourceEndX;
int lineSourceEndY;

// mode pixel arrays
PGraphics plantGraphics;
PGraphics sourceGraphics;
PGraphics growGraphics;

// some colors
int white = color(80, 125, 42);
int gray = color(125, 125, 125);
int black = color(0, 0, 0);
  
// declare buttons and scrollbars
Button plantButton, sourceButton, growButton, stopButton;
ScrollBar gravityBar, velocityBar, fluxBar, lifeBar;


public void setup() {
  sources = new ArrayList();
  allPixels = new ArrayList();
  sourceID = 0;
  isNewGrow = true;
  size(screen.width / 2, screen.height / 2);
  background(0);
  loadGraphics();
  textFont(font);
  createRibbon();
  frameRate(100);
  //restart();
}

public void draw() {
  if (mode == GROW_MODE) {
    runDLA();
    runPixels();
  }
  updateRibbon();
}

public void restart() {
}

public void loadGraphics() {
  plantGraphics = createGraphics(width, 4 * height / 5, P2D);
  sourceGraphics = createGraphics(width, 4 * height / 5, P2D);
  growGraphics = createGraphics(width, 4 * height / 5, P2D);
  plantGraphics.beginDraw();
  plantGraphics.fill(black);
  plantGraphics.rect(0, 0, plantGraphics.width, plantGraphics.height);
  plantGraphics.endDraw();
  sourceGraphics.beginDraw();
  sourceGraphics.fill(black);
  sourceGraphics.rect(0, 0, plantGraphics.width, plantGraphics.height);
  sourceGraphics.endDraw();
  growGraphics.beginDraw();
  growGraphics.fill(black);
  growGraphics.rect(0, 0, plantGraphics.width, plantGraphics.height);
  growGraphics.endDraw();
}
  
public void createRibbon() {
  noStroke();
  fill(5);
  rect(0, 4 * height / 5, width, 1 * height / 5);
  ribbonHeight = height / 5;
  buttonWidth = width / 10;
  buttonHeight = ribbonHeight / 4 - 1;
  margin = 5;
  plantButton = new Button("PLANT", 9 * buttonWidth + margin, height - 4 * ribbonHeight / 4,
                              buttonWidth - margin - 1, buttonHeight,
                              true);
  sourceButton = new Button("SOURCE", 9 * buttonWidth + margin, height - 3 * ribbonHeight / 4,
                              buttonWidth - margin - 1, buttonHeight,
                              true);
  growButton = new Button("GROW", 9 * buttonWidth + margin, height - 2 * ribbonHeight / 4,
                              buttonWidth - margin - 1, buttonHeight,
                              true);
  stopButton = new Button("STOP", 9 * buttonWidth + margin, height - 1 * ribbonHeight / 4,
                              buttonWidth - margin - 1, buttonHeight,
                              true);
  fluxBar = new ScrollBar(buttonWidth + margin, height - 4 * ribbonHeight / 4 + buttonHeight / 2, 
                              8 * buttonWidth, buttonHeight + 1, scrollLoose);
  velocityBar = new ScrollBar(buttonWidth + margin, height - 3 * ribbonHeight / 4 + buttonHeight / 2, 
                              8 * buttonWidth, buttonHeight + 1, scrollLoose);
  gravityBar = new ScrollBar(buttonWidth + margin, height - 2 * ribbonHeight / 4 + buttonHeight / 2, 
                              8 * buttonWidth, buttonHeight + 1, scrollLoose);
  lifeBar = new ScrollBar(buttonWidth + margin, height - 1 * ribbonHeight / 4 + buttonHeight / 2, 
                              8 * buttonWidth, buttonHeight + 1, scrollLoose);
  fill(111, 177, 255);
  text("FLUX", 0, height - 4 * ribbonHeight / 4, buttonWidth, buttonHeight);
  text("VELOCITY", 0, height - 3 * ribbonHeight / 4, buttonWidth, buttonHeight);
  text("GRAVITY", 0, height - 2 * ribbonHeight / 4, buttonWidth, buttonHeight);
  text("LIFE", 0, height - 1 * ribbonHeight / 4, buttonWidth, buttonHeight);
}

public void currentMode() {
  switch (mode) {
    case 0:
      displayCurrentMode("Plant Mode");
      break;
    case 1: 
      displayCurrentMode("Source Mode");
      break;
    case 2: 
      displayCurrentMode("Grow Mode");
      break;
    case 3: 
      displayCurrentMode("Stop Mode");
      break; 
  }
}

public void displayCurrentMode(String modeString) {
  fill(0);
  rect(width - buttonWidth, 0, buttonWidth, buttonHeight);
  fill(111, 177, 255);
  textAlign(RIGHT);
  text(modeString, width - buttonWidth, 0, buttonWidth, buttonHeight);
}

public void updateRibbon() {
  plantButton.update();
  sourceButton.update();
  growButton.update();
  stopButton.update();
  fluxBar.update();
  velocityBar.update();
  gravityBar.update();
  lifeBar.update();
  plantButton.display();
  sourceButton.display();
  growButton.display();
  stopButton.display();
  fluxBar.display();
  velocityBar.display();
  gravityBar.display();
  lifeBar.display();
  currentMode();
}

public void mouseClicked() {
  if (mouseY < 4 * height / 5) {
    checkMode();
  } else {
    checkButton();
  }  
}

public void checkMode() {
  if (mode == PLANT_MODE) {
    if (mouseY < 4 * height / 5 && mouseButton == LEFT) {
      stroke(white);
      point(mouseX, mouseY);
    } else if (mouseY < 4 * height / 5 && mouseButton == RIGHT) {
      setup();
    }
  } else if (mode == SOURCE_MODE) {
    
  } else if (mode == GROW_MODE) {
    mode = STOP_MODE;
  } else if (mode == STOP_MODE) {
    mode = GROW_MODE;
  }
}

public void checkButton() {
  if (plantButton.over()) {
    getImage();
    mode = PLANT_MODE;
    image(plantGraphics,0,0);
    isNewGrow = true;
  } else if (sourceButton.over()) {
    getImage();
    mode = SOURCE_MODE;
    image(sourceGraphics,0,0);
  } else if (growButton.over()) {
    getImage();
    mode = GROW_MODE;
    image(plantGraphics,0,0);
    if (isNewGrow) {
      isNewGrow = false;
      loadPixels();
      for (int row = 0; row < 4 * height / 5; row++) {
        for (int col = 0; col < width; col++) {
            allPixels.add(new ScreenPixel(row, col, pixels[row * width + col]));
            //println(((Pixel)DLAParticles.get(DLAParticles.size() - 1)).getRow() + " " + ((Pixel)DLAParticles.get(DLAParticles.size() - 1)).getCol() + " " + ((Pixel)DLAParticles.get(DLAParticles.size() - 1)).getColor());
        }
      }
    }
  } else if (stopButton.over()) {
    mode = STOP_MODE;
  }
}

public void getImage() {
  fill(0);
  rect(width - buttonWidth, 0, buttonWidth, buttonHeight);
  PImage capture = get(0, 0, width, 4 * height / 5);
  switch (mode) {
    case PLANT_MODE:
      plantGraphics.beginDraw();
      plantGraphics.image(capture, 0, 0);
      plantGraphics.endDraw();
      break;
    case SOURCE_MODE:
      sourceGraphics.beginDraw();
      sourceGraphics.image(capture, 0, 0);
      sourceGraphics.endDraw();
      break;
  }
}

public void mouseDragged () {
  if (mouseButton == LEFT) {
    if (mouseY < 4 * height / 5) {
      if (mode == PLANT_MODE) {
        stroke(white);
        point(mouseX, mouseY);
      }
    }
  }
}


public void mousePressed() { 
  if (mouseY < 4 * height / 5) {
    if (mode == SOURCE_MODE) {
      lineSourceBeginX = mouseX;
      lineSourceBeginY = mouseY;
      stroke(gray);
      point(lineSourceBeginX, lineSourceBeginY);
    }
  }
}

public void mouseReleased() {
  if (mouseY < 4 * height / 5) {
    if (mode == SOURCE_MODE) {
      lineSourceEndX = mouseX;
      lineSourceEndY = mouseY;
      stroke(gray);
      line(lineSourceBeginX, lineSourceBeginY, lineSourceEndX, lineSourceEndY);
      if ((lineSourceBeginX == lineSourceEndX) && (lineSourceBeginX == lineSourceEndX)) {
        PVector lineVector = new PVector(0, 0);
        sources.add(new DLA_Source(lineSourceBeginX, lineSourceBeginY, 
                                   fluxBar.value, velocityBar.value, gravityBar.value, lifeBar.value,
                                   false, lineVector, sourceID));
      }
      else {
        PVector lineVector = new PVector(lineSourceEndX - lineSourceBeginX, 
                                         lineSourceEndY - lineSourceBeginY);
        sources.add(new DLA_Source(lineSourceBeginX, lineSourceBeginY, 
                                   fluxBar.value, velocityBar.value, gravityBar.value, lifeBar.value,
                                   true, lineVector, sourceID));
      }
      sourceID++;
    }
  }
}

public void runDLA() {
  for(int i = 0; i < sources.size(); i++) {
    //println("sources.size() " + sources.size());
    currentSource = (DLA_Source) sources.get(i);
    currentSource.addParticle();
    currentSource.run();
    currentSource.removeDead();
  }
}

public void runPixels() {
  for(int i = 0; i < allPixels.size(); i++) {
    ScreenPixel currentPixel = new ScreenPixel();
    currentPixel = (ScreenPixel) allPixels.get(i);
    if (currentPixel.getColor() == white) {
      checkNeighbours(currentPixel);
      checkCurrentPixel(currentPixel);
      currentPixel.display();
    }
  }
}

public void checkNeighbours(ScreenPixel currentPixel) {
  for (int iX = -1; iX < 2; iX ++) {
    for (int iY = -1; iY < 2; iY ++) {
      ScreenPixel checkPixel = new ScreenPixel();
      checkPixel = (ScreenPixel) allPixels.get((currentPixel.getRow() + iY) * width + (currentPixel.getCol() + iX));
      if (checkPixel.getParticleID() >= 0 && checkPixel.getColor() != white) {
        if (abs(iY) == abs(iX) && random(0,1) > 0.1f) {return;}          
        if (checkPixel.getParticleID() >= ((DLA_Source) sources.get(checkPixel.getSourceID())).particles.size()) {return;}
        ((Particle) ((DLA_Source) sources.get(checkPixel.getSourceID())).particles.get(checkPixel.getParticleID())).life = 0;
        checkPixel.setSourceID(-1);
        checkPixel.setParticleID(-1);
        checkPixel.setColor(white);
      }
    }
  }
} 

public void checkCurrentPixel(ScreenPixel currentPixel) {
  if (currentPixel.getParticleID() >= 0) {
    if (currentPixel.getParticleID() >= ((DLA_Source) sources.get(currentPixel.getSourceID())).particles.size()) {return;}
    ((Particle) ((DLA_Source) sources.get(currentPixel.getSourceID())).particles.get(currentPixel.getParticleID())).life = 0;
    currentPixel.setSourceID(-1);
    currentPixel.setParticleID(-1);
  }
}

class Button
{
  int xPos, yPos;
  int bWidth, bHeight;
  boolean over;
  boolean isInteractive;
  String name;
  PFont fontButton = createFont("Showcard Gothic", 20);
  int clickedColor = color(111, 177, 255);

  Button (String name, int xPos, int yPos, int bWidth, int bHeight, boolean isInteractive) {
    this.name = name;
    this.xPos = xPos;
    this.yPos = yPos;
    this.bWidth = bWidth;
    this.bHeight = bHeight;
    this.isInteractive = isInteractive;
  }
  
  public void display() {
    textAlign(CENTER);
    textFont(fontButton);
    if(over && isInteractive) {
      stroke(225, 228, 242);
    }
    else {
      stroke(111, 177, 255);
    }
    fill(5);
    rect(xPos, yPos, bWidth, bHeight);
    if(over && isInteractive) {
      fill(225, 228, 242);
    }
    else {
      fill(111, 177, 255);
    }
    text(name, (xPos) + bWidth / 2, (yPos) + 2 * bHeight / 3);
  }
  
  public void update() {
    if(over()) {
      over = true;
    } else {
      over = false;
    }
  }
  
  public boolean over() {
    if(mouseX > xPos && mouseX < xPos+bWidth &&
    mouseY > yPos && mouseY < yPos+bHeight) {
      return true;
    } else {
      return false;
    }
  }
}
/*  
 * Core of this code is based on work of
 */
/*
 *  DLA - Crystal Simulation 
 *
 *  More Info: http://stungeye.com/archives/2007/10/usually_permane.php
 *
 *  Wally Glutton 2007
 *  http://stungeye.com
 *
 *  License: http://creativecommons.org/licenses/by-sa/3.0/
 *
 */


class DLA_Source {
  
  // DLA_Engine properties
  float originX;
  float originY;
  int flux;
  float velocity;
  float gravity;
  int life;
  boolean isLine;
  PVector lineVector;
  int sourceID;
    
  // DLA_Engine is
  ArrayList particles;
  
  // DLA_Engine Constructor
  DLA_Source(float originX, float originY, float flux, int velocity, int gravity, int life, boolean isLine, PVector lineVector, int sourceID) {
    this.originX = originX;
    this.originY = originY; 
    this.flux = round(flux / 20.0f);
    this.velocity = 1;
    this.gravity = (gravity - 50)/ 100.0f;
    this.life = life;
    this.isLine = isLine;
    this.lineVector = lineVector;
    this.particles = new ArrayList();
    this.sourceID = sourceID;
  }
  
  // add particles to DLA_Engine
  public void addParticle() {
    if(isLine) {
      for (int i=0; i < flux; i++) {
        float randomValue = random(lineVector.mag() + 0.01f) / lineVector.mag();
        float particleOriginX = originX + randomValue * lineVector.x;
        float particleOriginY = originY + randomValue * lineVector.y;
        float velocityVectorAngle = (this.lineVector.heading2D() > 0 ? 
                                     this.lineVector.heading2D() + PI / 2 :
                                     PI / 2 - abs(this.lineVector.heading2D()));

        PVector velocityVector = new PVector(this.velocity * cos(velocityVectorAngle),
                                             this.velocity * sin(velocityVectorAngle));
        this.particles.add(new Particle(particleOriginX, particleOriginY, velocityVector, gravity, life, this.sourceID, this.particles.size()));
      }
    } else {
      for (int i=0; i < flux; i++) {
        PVector velocityVector = new PVector(this.velocity * cos(random(0, 100000 * 2 * PI) / 100000.0f),
                                             this.velocity * sin(random(0, 100000 * 2 * PI) / 100000.0f));
        this.particles.add(new Particle(originX, originY, velocityVector, gravity, life, this.sourceID, this.particles.size()));
      }
    }
  }
  
  //remove dead particles of this source
  public void removeDead() {
    Particle p;
    int num_particles = this.particles.size();
    for (int i = num_particles - 1; i >= 0; i--) {
      p = (Particle) this.particles.get(i);
      if (p.life <= 0) {
        this.particles.remove(i);
      }
    }
  }
  
  // simulate particles of this source
  public void run() {
    Particle p;
    int num_particles = particles.size();
    for (int i = 0; i < num_particles; i++) {
      p = (Particle) particles.get(i);
      p.setID(i);
      p.moveParticle();
    }
  }
}

/*  
 * Core of this code is based on work of
 */
/*
 *  DLA - Crystal Simulation 
 *
 *  More Info: http://stungeye.com/archives/2007/10/usually_permane.php
 *
 *  Wally Glutton 2007
 *  http://stungeye.com
 *
 *  License: http://creativecommons.org/licenses/by-sa/3.0/
 *
 */
 

class Particle {
  
  // some colors
  int white = color(80, 125, 42);
  int sapGreen = color (80, 125, 42);
  int gray = color(125, 125, 125);
  int black = color(0, 0, 0);
  
  // Particle properties
  float positionX;
  float positionY;
  PVector velocityVector;
  PVector gravityVector;
  float life;
  int state;
  int sourceID;
  int particleID;
  
  // construct a particle
  Particle(float particleOriginX, float particleOriginY, PVector velocityVector, float gravity, int life, int sourceID, int particleID) {
    this.positionX = particleOriginX;
    this.positionY = particleOriginY;
    this.velocityVector = PVector.mult(velocityVector, random(0, 1));
    this.gravityVector = PVector.mult(new PVector(0, gravity / 2), random(0, 1));
    this.life = life * random(1, 5);
    this.state = 0;
    this.sourceID = sourceID;
    this.particleID = particleID;
  } 
  
  public void setID(int particleID) {
    this.particleID = particleID;
  }
  
  // check if particle is out of screen boundary
  public boolean outOfBoundary(float x, float y) {
    if ((round(x) < 0) || (round(x) > width)) return true;
    if ((round(y) < 0) || (round(y) > 4 * height / 5 - 2)) return true;
    return false;
  }
  
  // control movement of particle
  public void moveParticle() {
    ((ScreenPixel) allPixels.get(round(this.positionY) * width + round(this.positionX))).setSourceID(-1);
    ((ScreenPixel) allPixels.get(round(this.positionY) * width + round(this.positionX))).setParticleID(-1);
    this.velocityVector.add(gravityVector);
    this.positionX += velocityVector.x + random(-2,2);
    this.positionY += velocityVector.y + random(-2,2);
    this.life--;
    // remove particle if its out of boundary
    if (outOfBoundary(this.positionX, this.positionY)) {
      this.life = 0;
      return;
    }
    ((ScreenPixel) allPixels.get(round(this.positionY) * width + round(this.positionX))).setSourceID(this.sourceID);
    ((ScreenPixel) allPixels.get(round(this.positionY) * width + round(this.positionX))).setParticleID(this.particleID);
  }
}
class ScreenPixel {
  
  private final int row;
  private final int col;
  private int pixColor;
  private int sourceID;
  private int particleID;
  
  ScreenPixel() {
    this.row = -1;
    this.col = -1;
  }
  
  ScreenPixel(final int row, final int col, int pixColor) {
    this.row = row;
    this.col = col;
    this.pixColor = pixColor;
    this.sourceID = -1;
    this.particleID = -1;
  }
  
  public int getRow() {
    return this.row;
  }
  
  public int getCol() {
    return this.col;
  }
  
  public int getColor() {
    return this.pixColor;
  }
  
  public int getSourceID() {
    return this.sourceID;
  }
  
  public int getParticleID() {
    return this.particleID;
  }
  
  public void setColor(int pixColor) {
    this.pixColor = pixColor;
  }
  
  public void setSourceID(int sourceID) {
    this.sourceID = sourceID;
  }
  
  public void setParticleID(int particleID) {
    this.particleID = particleID;
  }
  
  public boolean checkPixel(int sourceColor) {
     if (this.pixColor == sourceColor) {
       return true;
     }
     return false;
  }
  
  public void display() {
    stroke(this.pixColor);
    point(this.col, this.row);
  }
    
}
class ScrollBar
{
  int swidth, sheight;    // width and height of bar
  int xpos, ypos;         // x and y position of bar
  float spos, newspos;    // x position of slider
  int sposMin, sposMax;   // max and min values of slider
  int loose;              // how loose/heavy
  boolean over;           // is the mouse over the slider?
  boolean locked;
  float ratio;
  int value;
  PFont fontBar = createFont("Showcard Gothic", 20);

  ScrollBar (int xp, int yp, int sw, int sh, int l) {
    swidth = sw;
    sheight = sh;
    int widthtoheight = sw - sh;
    ratio = (float)sw / (float)widthtoheight;
    xpos = xp;
    ypos = yp-sheight/2;
    spos = xpos;
    newspos = spos;
    sposMin = xpos;
    sposMax = xpos + swidth - sheight;
    loose = l;
    PFont fontBar = createFont("Stencil", 8);
  }

  public void update() {
    if(over()) {
      over = true;
    } else {
      over = false;
    }
    if(mousePressed && over) {
      locked = true;
    }
    if(!mousePressed) {
      locked = false;
    }
    if(locked) {
      newspos = constrain(mouseX-sheight/2, sposMin, sposMax);
    }
    if(abs(newspos - spos) > 1) {
      spos = spos + (newspos-spos)/loose;
    }
  }

  public int constrain(int val, int minv, int maxv) {
    return min(max(val, minv), maxv);
  }

  public boolean over() {
    if(mouseX > xpos && mouseX < xpos+swidth &&
    mouseY > ypos && mouseY < ypos+sheight) {
      return true;
    } else {
      return false;
    }
  }

  public void display() {
    noStroke();
    textAlign(CENTER);
    textFont(fontBar);
    fill(5);
    rect(xpos, ypos, swidth, sheight);
    value = round((spos - xpos) / (swidth - sheight) * 99);
    fill(111, 177, 255);
    rectMode(CORNERS);
    rect(xpos, ypos, spos, ypos + sheight);
    rectMode(CORNER);
    if(over) {
      fill(225, 228, 242);
    }
    else {
      fill(111, 177, 255);
    }
    text(value, (spos) + sheight / 2, (ypos) + 2 * sheight / 3);
  }

  public float getPos() {
    // Convert spos to be values between
    // 0 and the total width of the scrollbar
    return spos * ratio;
  }
}

  static public void main(String args[]) {
    PApplet.main(new String[] { "--bgcolor=#F0F0F0", "dlaplaygroundoptimized" });
  }
}
